﻿using fgxavi.Mvvm;
using Newtonsoft.Json;
using Newtonsoft.Json.Linq;
using System;
using System.Linq;
using System.Collections.Generic;
using System.Net.Http;
using System.Net.Http.Headers;
using System.Threading.Tasks;
using System.ComponentModel.DataAnnotations;

namespace Sigcat.Client.Model
{

    public abstract class DataServiceBase : IDataService
    {

        public async Task<string> GetResponse(string httpGetUrl)
        {
            var client = new HttpClient();
            string response = await client.GetStringAsync(httpGetUrl);
            return response;
        }

        public async Task<T> GetJsonObject<T>(string httpGetUrl)
        {
            var json = await GetResponse(httpGetUrl);
            var JObj = JObject.Parse(json).First.First["RootResults"];
            if (JObj == null)
                return default(T);
            var rootResults = JObj.First;
            var result = rootResults.ToObject<T>();
            return result;
        }

        public async Task<IEnumerable<T>> GetJsonObjects<T>(string httpGetUrl)
        {
            string json = await GetResponse(httpGetUrl);
            var rootResults = JObject.Parse(json).First.First["RootResults"];
            var result = rootResults.ToObject<IEnumerable<T>>();
            return result;
        }

        public async Task<string> PostResponse(string httpPostUri, StringContent content)
        {
            var client = new HttpClient();
            try
            {
                var response = await client.PostAsync(httpPostUri, content);
                var result = await response.Content.ReadAsStringAsync();
                return result;
            }
            catch (Exception)
            {
                return "0";
            }
        }

        public async Task<string> PostJsonRequest(string httpPostUri, string json)
        {
            var content = new StringContent(json);
            content.Headers.ContentType = new MediaTypeHeaderValue("application/json");
            var result = await PostResponse(httpPostUri, content);
            return result;
        }

        public string GetJsonSubmitChanges(object entity, DomSvcJsonOperation operation, object originalEntity = null)
        {
            JObject jsonChangeSetEntry = new JObject();

            // Obté la clau primària de l'Entity.
            var jsonEntityId = (from prop in entity.GetType().GetProperties()
                                where Attribute.GetCustomAttributes(prop, typeof(KeyAttribute)) != null
                                select prop.GetValue(entity)).FirstOrDefault();
            // Si no s'ha trobat la clau primària retorna nul.
            if (jsonEntityId == null)
                return null;
            // Afegeix les propietats Json necessàries pel servei WCF Domain Service.
            jsonChangeSetEntry.Add(new JProperty("Id", jsonEntityId));
            jsonChangeSetEntry.Add(new JProperty("Operation", operation));
            //jsonChangeSet.Add(new JProperty("OperationName", null));
            //jsonChangeSet.Add(new JProperty("OperationData", null));

            // Converteix l'objecte a un objecte Json.Net.
            var jsonEntity = JObject.FromObject(entity);
            // Obté els atributs que informen del nom de l'entitat
            // i l'espai de noms de l'entitat al servidor.
            var jsonEntityType = (from attr in Attribute.GetCustomAttributes(entity.GetType(), true)
                                  where attr is EntityAttribute
                                  select (attr as EntityAttribute)).FirstOrDefault();
            // Si no s'han trobat els atributs retorna nul.
            if (jsonEntityType == null)
                return null;
            // Afegeix la propietat __type necessària per la comunicació amb WCF domain service.
            var jsonPropertyEntityType = new JProperty("__type", string.Format("{0}:#{1}",
                jsonEntityType.Name, jsonEntityType.Namespace));
            jsonEntity.AddFirst(jsonPropertyEntityType);
            // Afegeix la propietat Json Entity.
            jsonChangeSetEntry.Add(new JProperty("Entity", jsonEntity));

            if (operation == DomSvcJsonOperation.Update)
            {
                // Converteix l'objecte a un objecte Json.Net.
                var jsonOriginalEntity = JObject.FromObject(originalEntity);
                // Afegeix la propietat __type necessària per la comunicació amb WCF domain service.
                jsonOriginalEntity.AddFirst(jsonPropertyEntityType);
                // Afegeix la propietat Json OriginalEntity.
                jsonChangeSetEntry.Add(new JProperty("OriginalEntity", jsonOriginalEntity));
            }

            JObject jsonChangeSet = new JObject(
                new JProperty("changeSet", new JArray(jsonChangeSetEntry))
                );

            // Retorna l'objecte Json.Net.
            return jsonChangeSet.ToString();
        }

    }

    #region Enum DomSvcJsonOperation

    public enum DomSvcJsonOperation
    {
        Insert = 2,
        Update = 3,
        Delete = 4
    }

    #endregion

}
