﻿using fgxavi.Mvvm;
using Newtonsoft.Json.Linq;
using Sigcat.Client.Model.Settings;
using System;
using System.Collections.Generic;
using System.Data;
using System.Linq;
using System.Threading.Tasks;

namespace Sigcat.Client.Model
{

    public class Repository<T> : DataServiceBase, IRepository<T> where T : ModelBase
    {
        public string EntityName { get; set; }
        public string EntitiesName { get; set; }
        public ServiceTypes ServiceType { get; set; }

        public async Task<string> GetHttpGetUrl()
        {
            ServiceModel svc = await DataServiceSettings.GetServiceModel(ServiceType);
            return DataServiceSettings.GetServiceUri(svc);
        }

        public Repository()
        {
            ServiceType = ServiceTypes.DomSvc;
            var attrib = (from attr in Attribute.GetCustomAttributes(typeof(T), true)
                          where attr is RepositoryAttribute
                          select (attr as RepositoryAttribute)).FirstOrDefault();
            if (attrib != null)
            {
                EntityName = attrib.EntityName;
                EntitiesName = attrib.EntitiesName;
            }
        }

        public async Task<IEnumerable<T>> Browse()
        {
            IEnumerable<T> entities = null;
            string httpGetUrl = await GetHttpGetUrl();
            httpGetUrl = string.Format("{0}/Get{1}", httpGetUrl, EntitiesName);
            // S'obté la resposta del servidor a aquesta adreça.
            entities = await GetJsonObjects<T>(httpGetUrl);
            return entities;
        }

        public async Task<T> Read(int id)
        {
            T entity = null;
            string httpGetUrl = await GetHttpGetUrl();
            httpGetUrl = string.Format("{0}/Get{1}?{1}id={2}", httpGetUrl, EntityName, id);
            // S'obté la resposta del servidor a aquesta adreça.
            entity = await GetJsonObject<T>(httpGetUrl);
            return entity;
        }

        public async Task<ChangeResult<T>> Edit(T entity)
        {
            string httpGetUrl = await GetHttpGetUrl();
            httpGetUrl = string.Format("{0}/SubmitChanges", httpGetUrl);
            string json = GetJsonSubmitChanges(entity, DomSvcJsonOperation.Update, entity.OriginalBackup);
            // S'obté la resposta del servidor a aquesta adreça.
            string jsonResult = await PostJsonRequest(httpGetUrl, json);
            var submitChangesResult = DeserializeJsonResult(jsonResult);
            return submitChangesResult;
        }

        protected ChangeResult<T> DeserializeJsonResult(string jsonResult)
        {
            ChangeResult<T> result = new ChangeResult<T>();
            var rootResults = JObject.Parse(jsonResult);
            var submitChangesResult = rootResults["SubmitChangesResult"];
            if (submitChangesResult != null)
                result.EntityResult = (from scr in submitChangesResult
                                select scr.ToObject<ChangeEntityResult<T>>()).FirstOrDefault();
            else
            {
                result.HasError = true;
                var errorCode = rootResults["ErrorCode"];
                if (errorCode != null)
                    result.ErrorResult = errorCode.ToObject<ChangeErrorResult>();
            }
            return result;
        }

        public async Task<ChangeResult<ModelBase>> SubmitChange(T entity)
        {
            ChangeResult<T> result = null;
            // Per a cadascún dels estats del model de dades
            // es desencadena l'event corresponent.
            switch (entity.State)
            {
                case DataRowState.Added:
                    result = await Add(entity);
                    break;
                case DataRowState.Modified:
                    result = await Edit(entity);
                    break;
                case DataRowState.Deleted:
                    result = await Delete(entity);
                    break;
                case DataRowState.Detached:
                case DataRowState.Unchanged:
                default:
                    break;
            }
            ChangeResult<ModelBase> resultGeneric = new ChangeResult<ModelBase>()
            {
                HasError = result.HasError,
                ErrorResult = result.ErrorResult
            };
            if (result.EntityResult != null)
                resultGeneric.EntityResult = new ChangeEntityResult<ModelBase>()
                {
                    Entity = result.EntityResult.Entity,
                    Id = result.EntityResult.Id,
                    Operation = result.EntityResult.Operation,
                    HasMemberChanges = result.EntityResult.HasMemberChanges,
                    ValidationErrors = result.EntityResult.ValidationErrors
                };
            return resultGeneric;
        }

        public async Task<ChangeResult<T>> Add(T entity)
        {
            string httpGetUrl = await GetHttpGetUrl();
            httpGetUrl = string.Format("{0}/SubmitChanges", httpGetUrl);
            string json = GetJsonSubmitChanges(entity, DomSvcJsonOperation.Insert);
            // S'obté la resposta del servidor a aquesta adreça.
            string jsonResult = await PostJsonRequest(httpGetUrl, json);
            var submitChangesResult = DeserializeJsonResult(jsonResult);
            return submitChangesResult;
        }

        public async Task<ChangeResult<T>> Delete(T entity)
        {
            string httpGetUrl = await GetHttpGetUrl();
            httpGetUrl = string.Format("{0}/SubmitChanges", httpGetUrl);
            string json = GetJsonSubmitChanges(entity, DomSvcJsonOperation.Delete);
            // S'obté la resposta del servidor a aquesta adreça.
            string jsonResult = await PostJsonRequest(httpGetUrl, json);
            var submitChangesResult = DeserializeJsonResult(jsonResult);
            return submitChangesResult;
        }
    }

}
