﻿using System.Threading.Tasks;

namespace Sigcat.Client.Model.Settings
{

    public class DesignDataServiceSettings : IDataServiceSettings
    {

        #region Service Task<SettingsModel> GetSettings()

        /// <summary>
        /// Obté el model <see cref="SettingsModel"/> de l'aplicació.
        /// </summary>
        /// <returns>
        /// Tasca amb el model de dades
        /// <see cref="SettingsModel"/> de l'aplicació.
        /// </returns>
        /// <remarks>
        /// Si no s'ha obtingut la configuració abans
        /// es recupera del fitxer de configuració.
        /// </remarks>
        public Task<SettingsModel> GetSettings()
        {
            return RestoreSettings();
        }

        #endregion

        #region Service Task<SettingsModel> RestoreSettings()

        /// <summary>
        /// Obté el model <see cref="SettingsModel"/> de l'arxiu XML
        /// de configuració de l'aplicació.
        /// </summary>
        /// <returns>
        /// Tasca amb el model de dades
        /// <see cref="SettingsModel"/> de l'aplicació.
        /// </returns>
        /// <remarks>
        /// Si no es pot recuperar el fitxer es retornen dades per defecte.
        /// </remarks>
        public Task<SettingsModel> RestoreSettings()
        {
            // Es retorna un model de dades per defecte.
            SettingsModel result = new SettingsModel()
            {
                AuthenticationService = new ServiceModel() { Path = "Authentication.svc" },
                DomainService = new ServiceModel() { Path = "Domain.svc" }
            };
            return Task.FromResult(result);
        }

        #endregion

        #region Service Task SaveSettings(SettingsModel settings)

        /// <summary>
        /// Emmagatzema el model de dades <see cref="SettingsModel"/>
        /// al fitxer de configuració Xml de l'aplicació.
        /// </summary>
        /// <param name="settings">
        /// Model de dades <see cref="SettingsModel"/>
        /// </param>
        /// <returns>Tasca sense resultat.</returns>
        public Task SaveSettings(SettingsModel settings)
        {
            return Task.FromResult(true);
        }

        #endregion

        #region Service Task<bool> CheckService(ServiceModel service, ServiceTypes serviceType)

        /// <summary>
        /// Comprova la connexió amb el servei <see cref="ServiceModel"/>.
        /// </summary>
        /// <param name="serviceType">
        /// Servei de l'aplicació al que es refereix el model de dades.
        /// </param>
        /// <returns>
        /// Vertader si es pot realitzar la connexió amb el servei.
        /// Fals si no es pot connectar amb el servei.
        /// </returns>
        public Task<bool> CheckService(ServiceTypes serviceType)
        {
            return Task.FromResult(true);
        }

        #endregion

        #region Service Task<string> GetServiceUri(ServiceTypes serviceType)

        /// <summary>
        /// Obté la cadena de connexió del servei <see cref="ServiceModel"/>.
        /// </summary>
        /// <param name="serviceType">
        /// Servei de l'aplicació al que es refereix el model de dades.
        /// </param>
        /// <returns>
        /// Cadena de connexió Uri del servei.
        /// </returns>
        public async Task<string> GetServiceUri(ServiceTypes serviceType)
        {
            SettingsModel settings = await GetSettings();
            ServiceModel service = DataServiceSettings.GetServiceModel(serviceType, settings);
            string httpGetUrl = DataServiceSettings.GetServiceUri(service);
            return httpGetUrl;
        }

        #endregion

    }

}
