﻿using System.Threading.Tasks;

namespace Sigcat.Client.Model.Settings
{
    public interface IDataServiceSettings
    {

        #region Service Task<SettingsModel> GetSettings()

        /// <summary>
        /// Obté el model <see cref="SettingsModel"/> de l'aplicació.
        /// </summary>
        /// <returns>
        /// Tasca amb el model de dades
        /// <see cref="SettingsModel"/> de l'aplicació.
        /// </returns>
        /// <remarks>
        /// Si no s'ha obtingut la configuració abans
        /// es recupera del fitxer de configuració.
        /// </remarks>
        Task<SettingsModel> GetSettings();

        #endregion

        #region Service Task<SettingsModel> RestoreSettings()

        /// <summary>
        /// Obté el model <see cref="SettingsModel"/> de l'arxiu XML
        /// de configuració de l'aplicació.
        /// </summary>
        /// <returns>
        /// Tasca amb el model de dades
        /// <see cref="SettingsModel"/> de l'aplicació.
        /// </returns>
        /// <remarks>
        /// Si no es pot recuperar el fitxer es retornen dades per defecte.
        /// </remarks>
        Task<SettingsModel> RestoreSettings();

        #endregion

        #region Service Task SaveSettings(SettingsModel settings)

        /// <summary>
        /// Emmagatzema el model de dades <see cref="SettingsModel"/>
        /// al fitxer de configuració Xml de l'aplicació.
        /// </summary>
        /// <param name="settings">
        /// Model de dades <see cref="SettingsModel"/>
        /// </param>
        /// <returns>Tasca sense resultat.</returns>
        Task SaveSettings(SettingsModel settings);

        #endregion

        #region Service Task<bool> CheckService(ServiceTypes serviceType)

        /// <summary>
        /// Comprova la connexió amb el servei <see cref="ServiceModel"/>.
        /// </summary>
        /// <param name="serviceType">
        /// Servei de l'aplicació al que es refereix el model de dades.
        /// </param>
        /// <returns>
        /// Vertader si es pot realitzar la connexió amb el servei.
        /// Fals si no es pot connectar amb el servei.
        /// </returns>
        Task<bool> CheckService(ServiceTypes serviceType);

        #endregion

        #region Service Task<string> GetServiceUri(ServiceTypes serviceType)

        /// <summary>
        /// Obté la cadena de connexió del servei <see cref="ServiceModel"/>.
        /// </summary>
        /// <param name="serviceType">
        /// Servei de l'aplicació al que es refereix el model de dades.
        /// </param>
        /// <returns>
        /// Cadena de connexió Uri del servei.
        /// </returns>
        Task<string> GetServiceUri(ServiceTypes serviceType);

        #endregion

    }

}
