﻿using fgxavi.Mvvm;
using Newtonsoft.Json;
using Newtonsoft.Json.Converters;
using System;
using System.Collections.Generic;
using System.ComponentModel.DataAnnotations;

namespace Sigcat.Client.Model.Settings
{

    public partial class ServiceModel : ModelBase
    {

        #region Hidden properties

        #region Property Name

        /// <summary>
        /// Membre privat de la propietat <see cref="Name" />.
        /// </summary>
        private string _Name = null;

        /// <summary>
        /// Estableix i obté la propietat <see cref="Name" />.
        /// Fa ús del mètode <see cref="SetField"/> per establir la propietat.
        /// </summary>
        public string Name
        {
            get { return _Name; }
            set { SetField(ref _Name, value); }
        }

        #endregion

        #region Property ServiceType

        /// <summary>
        /// Membre privat de la propietat <see cref="ServiceType" />.
        /// </summary>
        private ServiceTypes _ServiceType = ServiceTypes.DomSvc;

        /// <summary>
        /// Estableix i obté la propietat <see cref="ServiceType" />.
        /// Fa ús del mètode <see cref="SetField"/> per establir la propietat.
        /// </summary>
        public ServiceTypes ServiceType
        {
            get { return _ServiceType; }
            set { SetField(ref _ServiceType, value); }
        }

        #endregion

        #region Property Url

        /// <summary>
        /// Membre privat de la propietat <see cref="Url" />.
        /// </summary>
        private UriBuilder _Url = new UriBuilder();

        /// <summary>
        /// Estableix i obté la propietat <see cref="Url" />.
        /// Fa ús del mètode <see cref="SetField"/> per establir la propietat.
        /// </summary>
        public UriBuilder Url
        {
            get { return _Url; }
            set { SetField(ref _Url, value); }
        }

        #endregion

        #region Property Verified

        /// <summary>
        /// Membre privat de la propietat <see cref="Verified" />.
        /// </summary>
        private VerifiedStatus _Verified = VerifiedStatus.NotVerified;

        /// <summary>
        /// Estableix i obté la propietat <see cref="Verified" />.
        /// Fa ús del mètode <see cref="SetField"/> per establir la propietat.
        /// </summary>
        public VerifiedStatus Verified
        {
            get { return _Verified; }
            set { SetField(ref _Verified, value); }
        }

        #endregion

        #endregion

        #region Properties

        #region Property Scheme

        /// <summary>
        /// Membre privat amb la propietat <see cref="Type" />.
        /// </summary>
        private SchemeTypes _Scheme = SchemeTypes.http;

        /// <summary>
        /// Estableix i obté la propietat <see cref="Scheme" />.
        /// Fa ús del mètode <see cref="SetField"/> per establir la propietat.
        /// </summary>
        public SchemeTypes Scheme
        {
            get { return _Scheme; }
            set { SetField(ref _Scheme, value); }
        }

        #endregion

        #region Property Host

        /// <summary>
        /// Membre privat amb la propietat <see cref="Type" />.
        /// </summary>
        private string _Host = "localhost";

        /// <summary>
        /// Estableix i obté la propietat <see cref="Host" />.
        /// Fa ús del mètode <see cref="SetField"/> per establir la propietat.
        /// </summary>
        public string Host
        {
            get { return _Host; }
            set { SetField(ref _Host, value); }
        }

        #endregion

        #region Property Port

        /// <summary>
        /// Membre privat amb la propietat <see cref="Type" />.
        /// </summary>
        private int _Port = 22888;

        /// <summary>
        /// Estableix i obté la propietat <see cref="Port" />.
        /// Fa ús del mètode <see cref="SetField"/> per establir la propietat.
        /// </summary>
        public int Port
        {
            get { return _Port; }
            set { SetField(ref _Port, value); }
        }

        #endregion

        #region Property Path

        /// <summary>
        /// Membre privat amb la propietat <see cref="Type" />.
        /// </summary>
        private string _Path = "{service}";

        /// <summary>
        /// Estableix i obté la propietat <see cref="Path" />.
        /// Fa ús del mètode <see cref="SetField"/> per establir la propietat.
        /// </summary>
        public string Path
        {
            get { return _Path; }
            set { SetField(ref _Path, value); }
        }

        #endregion

        #region Property EndPoint

        /// <summary>
        /// Membre privat amb la propietat <see cref="Type" />.
        /// </summary>
        private EndPoints _EndPoint = EndPoints.Json;

        /// <summary>
        /// Estableix i obté la propietat <see cref="EndPoint" />.
        /// Fa ús del mètode <see cref="SetField"/> per establir la propietat.
        /// </summary>
        public EndPoints EndPoint
        {
            get { return _EndPoint; }
            set { SetField(ref _EndPoint, value); }
        }

        #endregion

        #endregion

    }

    [MetadataType(typeof(Metadata))]
    public partial class ServiceModel
    {
        internal sealed class Metadata
        {

            #region Hidden

            [Required]
            [JsonIgnore]
            public string Name { get; set; }
            [Required]
            [JsonIgnore]
            public ServiceTypes ServiceType { get; set; }
            [Required]
            [JsonIgnore]
            public UriBuilder Url { get; set; }
            [Required]
            [JsonIgnore]
            public VerifiedStatus Verified { get; set; }

            #endregion

            [Required]
            [JsonProperty]
            [JsonConverter(typeof(StringEnumConverter))]
            public SchemeTypes Scheme { get; set; }
            [Required]
            [JsonProperty]
            public string Host { get; set; }
            [Required]
            [Range(0, 65535)]
            [JsonProperty]
            public int Port { get; set; }
            [Required]
            [JsonProperty]
            public string Path { get; set; }
            [Required]
            [JsonProperty]
            [JsonConverter(typeof(StringEnumConverter))]
            public EndPoints EndPoint { get; set; }
        }
    }

    #region EnumType ServiceTypes

    /// <summary>
    /// Nom tots els serveis de l'aplicació.
    /// </summary>
    /// <remarks>
    /// Han de correspondre amb el nom de la classe del servei.
    /// </remarks>
    public enum ServiceTypes
    {
        AuthSvc,
        DomSvc
    }

    #endregion

    #region EnumType SchemeTypes

    /// <summary>
    /// Protocols del servidor web.
    /// </summary>
    public enum SchemeTypes
    {
        http,
        https
    }

    #endregion

    #region EnumType EndPoints

    /// <summary>
    /// Tipus de servei web.
    /// </summary>
    public enum EndPoints
    {
        Json,
        OData,
        Soap
    }

    #endregion

    #region EnumType VerifiedStatus

    /// <summary>
    /// Estat de verificació de l'establiment de la connexió.
    /// </summary>
    public enum VerifiedStatus
    {
        NotVerified,
        Establishing,
        Verified,
        Failed
    }

    #endregion

}
