﻿using ESRI.ArcGIS.Client;
using ESRI.ArcGIS.Client.Geometry;
using ESRI.ArcGIS.Client.Toolkit.DataSources;
using GalaSoft.MvvmLight;
using Newtonsoft.Json.Linq;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Windows;
using Sigcat.Client.Model;

namespace Sigcat.Client.ViewModel
{
    /// <summary>
    /// This class contains properties that a View can data bind to.
    /// <para>
    /// See http://www.galasoft.ch/mvvm
    /// </para>
    /// </summary>
    public class EsriMapViewModel : ViewModelBase
    {

        /// <summary>
        /// Initializes a new instance of the EsriMapViewModel class.
        /// </summary>
        public EsriMapViewModel()
        {
            Layers.Add(new ArcGISTiledMapServiceLayer()
            {
                ID = "World_Street_Map",
                Url = @"http://services.arcgisonline.com/ArcGIS/rest/services/World_Street_Map/MapServer"
            });
            LayerOverviewMap = new ArcGISTiledMapServiceLayer()
            {
                ID = "WorldStreetMapOverview",
                Url = @"http://services.arcgisonline.com/ArcGIS/rest/services/World_Street_Map/MapServer"
            };
            if (IsInDesignMode)
            {
                // Code runs in Blend --> create design time data.
                this.IsNavigationVisible = true;
                this.IsLegendVisible = true;
                this.IsScaleLineVisible = true;
                this.IsLayersVisible = true;
                this.IsOverviewMapVisible = true;
            }
            else
            {
                GroupLayer LayersICC = new GroupLayer()
                {
                    ID = "Institut Cartogràfic de Catalunya"
                };
                Layer l = new WmsLayer()
                {
                    ID = "Web Map Services",
                    Layers = new string[] { "mtc1000m", "mtc500m", "mtc250m", "mtc50m", "mtc25m", "mtc10m", "mtc5m" }, // "sat250m", 
                    Url = @"http://shagrat.icc.es/lizardtech/iserv/ows?",
                    SupportedSpatialReferenceIDs = new int[] { 25831, 32631, 4230, 4258, 4326 }
                };
                //typeof(Layer).GetProperty("SpatialReference").SetValue(l, new SpatialReference(25831));
                typeof(Layer).GetProperty("FullExtent").SetValue(l, new Envelope(259000, 4487000, 530000, 4753000));
                //LayersICC.ChildLayers.Add(l);
                //LayersICC.ChildLayers.Add(new WmsLayer()
                //{
                //    ID = "Dept. Territori i Sostenibilitat",
                //    Url = @"http://tes.gencat.cat/webmap/dgcWMS/request.aspx"
                //});
                GroupLayer LayersArcGisICC = new GroupLayer()
                {
                    ID = "ArcGIS Services"
                };
                WebClient client = new WebClient();
                string result = client.DownloadString(new Uri(@"http://galileo.icc.cat/arcgis/rest/services?f=pjson"));
                JObject json = JObject.Parse(result);
                List<string> services = json["services"].Select(o => (string)o["name"]).ToList();
                foreach (string service in services)
                {
                    ArcGISDynamicMapServiceLayer layer = new ArcGISDynamicMapServiceLayer()
                    {
                        ID = service,
                        Visible = false,
                        Url = string.Format(@"http://galileo.icc.cat/arcgis/rest/services/{0}/MapServer", service)
                    };
                    try
                    {
                        result = client.DownloadString(new Uri(string.Format("{0}?f=json", layer.Url)));
                        json = JObject.Parse(result);
                        if (json["error"] != null)
                            continue;
                        LayersArcGisICC.ChildLayers.Add(layer);
                    }
                    catch (Exception)
                    {
                        continue;
                    }
                }
                LayersICC.ChildLayers.Add(LayersArcGisICC);
                Layers.Add(LayersICC);
                // Capa de dibuix.
                Layers.Add(new GraphicsLayer()
                {
                    ID = "MyGraphicsLayer"
                });
            }
        }

        #region Property NavigationVisibility

        /// <summary>
        /// The <see cref="NavigationVisibility" /> property's name.
        /// </summary>
        public const string NavigationVisibilityPropertyName = "NavigationVisibility";

        /// <summary>
        /// Sets and gets the NavigationVisibility property.
        /// Changes to that property's value raise the PropertyChanged event. 
        /// </summary>
        public Visibility NavigationVisibility
        {
            get
            {
                if (IsNavigationVisible)
                    return Visibility.Visible;
                return Visibility.Collapsed;
            }
        }

        #endregion

        #region Property IsNavigationVisible

        /// <summary>
        /// The <see cref="IsNavigationVisible" /> property's name.
        /// </summary>
        public const string IsNavigationVisiblePropertyName = "IsNavigationVisible";

        private bool _IsNavigationVisible = false;

        /// <summary>
        /// Sets and gets the IsNavigationVisible property.
        /// Changes to that property's value raise the PropertyChanged event. 
        /// </summary>
        public bool IsNavigationVisible
        {
            get
            {
                return _IsNavigationVisible;
            }

            set
            {
                if (_IsNavigationVisible == value)
                {
                    return;
                }

                RaisePropertyChanging(IsNavigationVisiblePropertyName);
                RaisePropertyChanging(NavigationVisibilityPropertyName);
                _IsNavigationVisible = value;
                RaisePropertyChanged(NavigationVisibilityPropertyName);
                RaisePropertyChanged(IsNavigationVisiblePropertyName);
            }
        }
        
        #endregion

        #region Property LegendVisibility

        /// <summary>
        /// The <see cref="LegendVisibility" /> property's name.
        /// </summary>
        public const string LegendVisibilityPropertyName = "LegendVisibility";

        /// <summary>
        /// Sets and gets the LegendVisibility property.
        /// Changes to that property's value raise the PropertyChanged event. 
        /// </summary>
        public Visibility LegendVisibility
        {
            get
            {
                if (IsLegendVisible)
                    return Visibility.Visible;
                return Visibility.Collapsed;
            }
        }

        #endregion

        #region Property IsLegendVisible

        /// <summary>
        /// The <see cref="IsLegendVisible" /> property's name.
        /// </summary>
        public const string IsLegendVisiblePropertyName = "IsLegendVisible";

        private bool _IsLegendVisible = false;

        /// <summary>
        /// Sets and gets the IsLegendVisible property.
        /// Changes to that property's value raise the PropertyChanged event. 
        /// </summary>
        public bool IsLegendVisible
        {
            get
            {
                return _IsLegendVisible;
            }

            set
            {
                if (_IsLegendVisible == value)
                {
                    return;
                }

                RaisePropertyChanging(IsLegendVisiblePropertyName);
                RaisePropertyChanging(LegendVisibilityPropertyName);
                _IsLegendVisible = value;
                RaisePropertyChanged(LegendVisibilityPropertyName);
                RaisePropertyChanged(IsLegendVisiblePropertyName);
            }
        }

        #endregion

        #region Property LayersVisibility

        /// <summary>
        /// The <see cref="LayersVisibility" /> property's name.
        /// </summary>
        public const string LayersVisibilityPropertyName = "LayersVisibility";

        /// <summary>
        /// Sets and gets the LayersVisibility property.
        /// Changes to that property's value raise the PropertyChanged event. 
        /// </summary>
        public Visibility LayersVisibility
        {
            get
            {
                if (IsLayersVisible)
                    return Visibility.Visible;
                return Visibility.Collapsed;
            }
        }

        #endregion

        #region Property IsLayersVisible

        /// <summary>
        /// The <see cref="IsLayersVisible" /> property's name.
        /// </summary>
        public const string IsLayersVisiblePropertyName = "IsLayersVisible";

        private bool _IsLayersVisible = false;

        /// <summary>
        /// Sets and gets the IsLayersVisible property.
        /// Changes to that property's value raise the PropertyChanged event. 
        /// </summary>
        public bool IsLayersVisible
        {
            get
            {
                return _IsLayersVisible;
            }

            set
            {
                if (_IsLayersVisible == value)
                {
                    return;
                }

                RaisePropertyChanging(IsLayersVisiblePropertyName);
                RaisePropertyChanging(LayersVisibilityPropertyName);
                _IsLayersVisible = value;
                RaisePropertyChanged(LayersVisibilityPropertyName);
                RaisePropertyChanged(IsLayersVisiblePropertyName);
            }
        }

        #endregion

        #region Property ScaleLineVisibility

        /// <summary>
        /// The <see cref="ScaleLineVisibility" /> property's name.
        /// </summary>
        public const string ScaleLineVisibilityPropertyName = "ScaleLineVisibility";

        /// <summary>
        /// Sets and gets the ScaleLineVisibility property.
        /// Changes to that property's value raise the PropertyChanged event. 
        /// </summary>
        public Visibility ScaleLineVisibility
        {
            get
            {
                if (IsScaleLineVisible)
                    return Visibility.Visible;
                return Visibility.Collapsed;
            }
        }

        #endregion

        #region Property IsScaleLineVisible

        /// <summary>
        /// The <see cref="IsScaleLineVisible" /> property's name.
        /// </summary>
        public const string IsScaleLineVisiblePropertyName = "IsScaleLineVisible";

        private bool _IsScaleLineVisible = false;

        /// <summary>
        /// Sets and gets the IsScaleLineVisible property.
        /// Changes to that property's value raise the PropertyChanged event. 
        /// </summary>
        public bool IsScaleLineVisible
        {
            get
            {
                return _IsScaleLineVisible;
            }

            set
            {
                if (_IsScaleLineVisible == value)
                {
                    return;
                }

                RaisePropertyChanging(IsScaleLineVisiblePropertyName);
                RaisePropertyChanging(ScaleLineVisibilityPropertyName);
                _IsScaleLineVisible = value;
                RaisePropertyChanged(ScaleLineVisibilityPropertyName);
                RaisePropertyChanged(IsScaleLineVisiblePropertyName);
            }
        }

        #endregion

        #region Property OverviewMapVisibility

        /// <summary>
        /// The <see cref="OverviewMapVisibility" /> property's name.
        /// </summary>
        public const string OverviewMapVisibilityPropertyName = "OverviewMapVisibility";

        /// <summary>
        /// Sets and gets the OverviewMapVisibility property.
        /// Changes to that property's value raise the PropertyChanged event. 
        /// </summary>
        public Visibility OverviewMapVisibility
        {
            get
            {
                if (IsOverviewMapVisible)
                    return Visibility.Visible;
                return Visibility.Collapsed;
            }
        }

        #endregion

        #region Property IsOverviewMapVisible

        /// <summary>
        /// The <see cref="IsOverviewMapVisible" /> property's name.
        /// </summary>
        public const string IsOverviewMapVisiblePropertyName = "IsOverviewMapVisible";

        private bool _IsOverviewMapVisible = false;

        /// <summary>
        /// Sets and gets the IsOverviewMapVisible property.
        /// Changes to that property's value raise the PropertyChanged event. 
        /// </summary>
        public bool IsOverviewMapVisible
        {
            get
            {
                return _IsOverviewMapVisible;
            }

            set
            {
                if (_IsOverviewMapVisible == value)
                {
                    return;
                }

                RaisePropertyChanging(IsOverviewMapVisiblePropertyName);
                RaisePropertyChanging(OverviewMapVisibilityPropertyName);
                _IsOverviewMapVisible = value;
                RaisePropertyChanged(OverviewMapVisibilityPropertyName);
                RaisePropertyChanged(IsOverviewMapVisiblePropertyName);
            }
        }

        #endregion

        #region Property Layers

        /// <summary>
        /// The <see cref="Layers" /> property's name.
        /// </summary>
        public const string LayersPropertyName = "Layers";

        private LayerCollection _Layers = new LayerCollection();

        /// <summary>
        /// Sets and gets the Layers property.
        /// Changes to that property's value raise the PropertyChanged event. 
        /// </summary>
        public LayerCollection Layers
        {
            get
            {
                return _Layers;
            }

            set
            {
                if (_Layers == value)
                {
                    return;
                }

                RaisePropertyChanging(LayersPropertyName);
                RaisePropertyChanging(LayerIDsPropertyName);
                _Layers = value;
                RaisePropertyChanged(LayerIDsPropertyName);
                RaisePropertyChanged(LayersPropertyName);
            }
        }
        
        #endregion

        #region Property LayerIDs

        /// <summary>
        /// The <see cref="LayerIDs" /> property's name.
        /// </summary>
        public const string LayerIDsPropertyName = "LayerIDs";

        /// <summary>
        /// Sets and gets the LayerIDs property.
        /// Changes to that property's value raise the PropertyChanged event. 
        /// </summary>
        public string LayerIDs
        {
            get
            {
                string result = "";
                foreach (Layer l in Layers)
                    result += string.Format("{0}, ", l.ID);
                return result;
            }
        }

        #endregion

        #region Property LayerOverviewMap

        /// <summary>
        /// The <see cref="LayerOverviewMap" /> property's name.
        /// </summary>
        public const string LayerOverviewMapPropertyName = "LayerOverviewMap";

        private Layer _LayerOverviewMap;

        /// <summary>
        /// Sets and gets the LayerOverviewMap property.
        /// Changes to that property's value raise the PropertyChanged event. 
        /// </summary>
        public Layer LayerOverviewMap
        {
            get
            {
                return _LayerOverviewMap;
            }

            set
            {
                if (_LayerOverviewMap == value)
                {
                    return;
                }

                RaisePropertyChanging(LayerOverviewMapPropertyName);
                _LayerOverviewMap = value;
                RaisePropertyChanged(LayerOverviewMapPropertyName);
            }
        }

        #endregion

    }
}
