﻿using ESRI.ArcGIS.Client;
using ESRI.ArcGIS.Client.Symbols;
//using fgxavi.Mvvm;
using GalaSoft.MvvmLight;
using GalaSoft.MvvmLight.Command;
using System.Windows.Media;

namespace Sigcat.Client.ViewModel
{
    /// <summary>
    /// This class contains properties that a View can data bind to.
    /// <para>
    /// See http://www.galasoft.ch/mvvm
    /// </para>
    /// </summary>
    public class EsriSymbolSelectorViewModel : ViewModelBase
    {

        #region Enumeration EsriSymbolModeType

        public enum EsriSymbolModeType
        {
            None,
            Point,
            Line,
            Polygon
        }

        #endregion

        #region Constructor

        /// <summary>
        /// Initializes a new instance of the EsriSymbolSelectorViewModel class.
        /// </summary>
        public EsriSymbolSelectorViewModel()
        {
            if (IsInDesignMode)
            {
                IsPointModeType = true;
                IsLineModeType = true;
                IsPolygonModeType = true;
            }
        }

        #endregion

        #region Property Mode

        /// <summary>
        /// The <see cref="Mode" /> property's name.
        /// </summary>
        public const string ModePropertyName = "Mode";

        private DrawMode _mode = DrawMode.None;

        /// <summary>
        /// Sets and gets the Mode property.
        /// Changes to that property's value raise the PropertyChanged event. 
        /// </summary>
        public DrawMode Mode
        {
            get
            {
                return _mode;
            }

            set
            {
                if (_mode == value)
                {
                    return;
                }

                RaisePropertyChanging(ModePropertyName);
                _mode = value;
                RaisePropertyChanged(ModePropertyName);

                // Canvia el tipus de símbol seleccionat.
                switch (_mode)
                {
                    case DrawMode.Point:
                        ModeType = EsriSymbolModeType.Point; break;
                    case DrawMode.Polyline:
                    case DrawMode.Freehand:
                        ModeType = EsriSymbolModeType.Line; break;
                    case DrawMode.Triangle:
                    case DrawMode.Rectangle:
                    case DrawMode.Circle:
                    case DrawMode.Ellipse:
                    case DrawMode.Arrow:
                        ModeType = EsriSymbolModeType.Polygon; break;
                    default:
                        ModeType = EsriSymbolModeType.None; break;
                }
            }
        }

        #endregion

        #region Property ModeType

        /// <summary>
        /// The <see cref="ModeType" /> property's name.
        /// </summary>
        public const string ModeTypePropertyName = "ModeType";

        private EsriSymbolModeType _ModeType = EsriSymbolModeType.Point;

        /// <summary>
        /// Sets and gets the ModeType property.
        /// Changes to that property's value raise the PropertyChanged event. 
        /// </summary>
        public EsriSymbolModeType ModeType
        {
            get
            {
                return _ModeType;
            }

            set
            {
                if (_ModeType == value)
                {
                    return;
                }

                RaisePropertyChanging(ModeTypePropertyName);
                _ModeType = value;
                RaisePropertyChanged(ModeTypePropertyName);

                // S'actualitzen les propietats booleanes.
                IsPointModeType = _ModeType == EsriSymbolModeType.Point;
                IsLineModeType = _ModeType == EsriSymbolModeType.Line;
                IsPolygonModeType = _ModeType == EsriSymbolModeType.Polygon;
            }
        }

        #endregion

        #region Point properties

        #region Property IsPointModeType

        /// <summary>
        /// The <see cref="IsPointModeType" /> property's name.
        /// </summary>
        public const string IsPointModeTypePropertyName = "IsPointModeType";

        private bool _isPointModeType = false;

        /// <summary>
        /// Sets and gets the IsPointModeType property.
        /// Changes to that property's value raise the PropertyChanged event. 
        /// </summary>
        public bool IsPointModeType
        {
            get
            {
                return _isPointModeType;
            }

            private set
            {
                if (_isPointModeType == value)
                {
                    return;
                }

                RaisePropertyChanging(IsPointModeTypePropertyName);
                _isPointModeType = value;
                RaisePropertyChanged(IsPointModeTypePropertyName);
            }
        }

        #endregion

        #region Property Point

        /// <summary>
        /// The <see cref="PointViewModel" /> property's name.
        /// </summary>
        public const string PointPropertyName = "Point";

        private PointViewModel _point = new PointViewModel();

        /// <summary>
        /// Sets and gets the Point property.
        /// Changes to that property's value raise the PropertyChanged event. 
        /// </summary>
        public PointViewModel Point
        {
            get
            {
                return _point;
            }

            set
            {
                if (_point == value)
                {
                    return;
                }

                RaisePropertyChanging(PointPropertyName);
                _point = value;
                RaisePropertyChanged(PointPropertyName);
            }
        }

        #endregion

        #endregion

        #region Line properties

        #region Property IsLineModeType

        /// <summary>
        /// The <see cref="IsLineModeType" /> property's name.
        /// </summary>
        public const string IsLineModeTypePropertyName = "IsLineModeType";

        private bool _isLineModeType = false;

        /// <summary>
        /// Sets and gets the IsLineModeType property.
        /// Changes to that property's value raise the PropertyChanged event. 
        /// </summary>
        public bool IsLineModeType
        {
            get
            {
                return _isLineModeType;
            }

            private set
            {
                if (_isLineModeType == value)
                {
                    return;
                }

                RaisePropertyChanging(IsLineModeTypePropertyName);
                _isLineModeType = value;
                RaisePropertyChanged(IsLineModeTypePropertyName);
            }
        }

        #endregion

        #region Property Line

        /// <summary>
        /// The <see cref="LineViewModel" /> property's name.
        /// </summary>
        public const string LinePropertyName = "Line";

        private LineViewModel _line = new LineViewModel();

        /// <summary>
        /// Sets and gets the Line property.
        /// Changes to that property's value raise the PropertyChanged event. 
        /// </summary>
        public LineViewModel Line
        {
            get
            {
                return _line;
            }

            set
            {
                if (_line == value)
                {
                    return;
                }

                RaisePropertyChanging(LinePropertyName);
                _line = value;
                RaisePropertyChanged(LinePropertyName);
            }
        }

        #endregion

        #endregion

        #region Polygon properties

        #region Property IsPolygonModeType

        /// <summary>
        /// The <see cref="IsPolygonModeType" /> property's name.
        /// </summary>
        public const string IsPolygonModeTypePropertyName = "IsPolygonModeType";

        private bool _isPolygonModeType = false;

        /// <summary>
        /// Sets and gets the IsPolygonModeType property.
        /// Changes to that property's value raise the PropertyChanged event. 
        /// </summary>
        public bool IsPolygonModeType
        {
            get
            {
                return _isPolygonModeType;
            }

            private set
            {
                if (_isPolygonModeType == value)
                {
                    return;
                }

                RaisePropertyChanging(IsPolygonModeTypePropertyName);
                _isPolygonModeType = value;
                RaisePropertyChanged(IsPolygonModeTypePropertyName);
            }
        }

        #endregion

        #region Property Polygon

        /// <summary>
        /// The <see cref="PolygonViewModel" /> property's name.
        /// </summary>
        public const string PolygonPropertyName = "Polygon";

        private PolygonViewModel _polygon = new PolygonViewModel();

        /// <summary>
        /// Sets and gets the Polygon property.
        /// Changes to that property's value raise the PropertyChanged event. 
        /// </summary>
        public PolygonViewModel Polygon
        {
            get
            {
                return _polygon;
            }

            set
            {
                if (_polygon == value)
                {
                    return;
                }

                RaisePropertyChanging(PolygonPropertyName);
                _polygon = value;
                RaisePropertyChanged(PolygonPropertyName);
            }
        }

        #endregion

        #endregion

        #region Command SelectModeCommand

        private RelayCommand<DrawMode> _selectModeCommand;

        /// <summary>
        /// Gets the SelectModeCommand.
        /// </summary>
        public RelayCommand<DrawMode> SelectModeCommand
        {
            get
            {
                return _selectModeCommand
                    ?? (_selectModeCommand = new RelayCommand<DrawMode>(
                        mode => Mode = mode,
                        mode => mode != Mode));
            }
        }

        #endregion

        //#region Property EsriSymbol

        // <summary>
        // Membre privat de la propietat <see cref="EsriSymbol" />.
        // </summary>
        //private Symbol _EsriSymbol = null;

        // <summary>
        // Estableix i obté la propietat <see cref="EsriSymbol" />.
        // Fa ús del mètode <see cref="SetField"/> per establir la propietat.
        // </summary>
        //public Symbol EsriSymbol
        //{
        //    get { return _EsriSymbol; }
        //    set { SetField<Symbol>(ref _EsriSymbol, value); }
        //}

        //#endregion

        //protected override void RaisePropertyChanged(string propertyName)
        //{
        //    base.RaisePropertyChanged(propertyName);
        //    if (propertyName != "EsriSymbol")
        //    {
        //        Symbol symbol = null;
        //        switch (ModeType)
        //        {
        //            case EsriSymbolModeType.Point:
        //                break;
        //            case EsriSymbolModeType.Line:

        //                break;
        //            case EsriSymbolModeType.Polygon:
        //                break;
        //            case EsriSymbolModeType.None:
        //            default:
        //                break;
        //        }
        //        EsriSymbol = symbol;
        //    }
        //}

        /// <summary>
        /// This class contains properties that a View can data bind to.
        /// <para>
        /// See http://www.galasoft.ch/mvvm
        /// </para>
        /// </summary>
        public class PointViewModel : ViewModelBase
        {

            #region Property Color

            /// <summary>
            /// The <see cref="Color" /> property's name.
            /// </summary>
            public const string ColorPropertyName = "Color";

            private Color _Color = Colors.Black;

            /// <summary>
            /// Sets and gets the Color property.
            /// Changes to that property's value raise the PropertyChanged event. 
            /// </summary>
            public Color Color
            {
                get
                {
                    return _Color;
                }

                set
                {
                    if (_Color == value)
                    {
                        return;
                    }

                    RaisePropertyChanging(ColorPropertyName);
                    _Color = value;
                    RaisePropertyChanged(ColorPropertyName);
                }
            }

            #endregion

            #region Property Width

            /// <summary>
            /// The <see cref="Width" /> property's name.
            /// </summary>
            public const string WidthPropertyName = "Width";

            private double _Width = 10;

            /// <summary>
            /// Sets and gets the Width property.
            /// Changes to that property's value raise the PropertyChanged event. 
            /// </summary>
            public double Width
            {
                get
                {
                    return _Width;
                }

                set
                {
                    if (_Width == value)
                    {
                        return;
                    }

                    RaisePropertyChanging(WidthPropertyName);
                    _Width = value;
                    RaisePropertyChanged(WidthPropertyName);
                }
            }

            #endregion

            #region Property Style

            /// <summary>
            /// The <see cref="Style" /> property's name.
            /// </summary>
            public const string StylePropertyName = "Style";

            private SimpleMarkerSymbol.SimpleMarkerStyle _Style = SimpleMarkerSymbol.SimpleMarkerStyle.Circle;

            /// <summary>
            /// Sets and gets the Style property.
            /// Changes to that property's value raise the PropertyChanged event. 
            /// </summary>
            public SimpleMarkerSymbol.SimpleMarkerStyle Style
            {
                get
                {
                    return _Style;
                }

                set
                {
                    if (_Style == value)
                    {
                        return;
                    }

                    RaisePropertyChanging(StylePropertyName);
                    _Style = value;
                    RaisePropertyChanged(StylePropertyName);
                }
            }

            #endregion

            #region Command SelectStyleCommand

            private RelayCommand<string> _selectStyleCommand;

            /// <summary>
            /// Gets the SelectStyleCommand.
            /// </summary>
            public RelayCommand<string> SelectStyleCommand
            {
                get
                {
                    return _selectStyleCommand
                        ?? (_selectStyleCommand = new RelayCommand<string>(
                            style => Style = (SimpleMarkerSymbol.SimpleMarkerStyle)
                                (System.Enum.Parse(typeof(SimpleMarkerSymbol.SimpleMarkerStyle), style, true)
                                ?? SimpleMarkerSymbol.SimpleMarkerStyle.Circle),
                            style => (SimpleMarkerSymbol.SimpleMarkerStyle)
                                System.Enum.Parse(typeof(SimpleMarkerSymbol.SimpleMarkerStyle), style, true)
                                != Style));
                }
            }

            #endregion

        }

        /// <summary>
        /// This class contains properties that a View can data bind to.
        /// <para>
        /// See http://www.galasoft.ch/mvvm
        /// </para>
        /// </summary>
        public class LineViewModel : ViewModelBase
        {

            #region Property Color

            /// <summary>
            /// The <see cref="Color" /> property's name.
            /// </summary>
            public const string ColorPropertyName = "Color";

            private Color _Color = Colors.Black;

            /// <summary>
            /// Sets and gets the Color property.
            /// Changes to that property's value raise the PropertyChanged event. 
            /// </summary>
            public Color Color
            {
                get
                {
                    return _Color;
                }

                set
                {
                    if (_Color == value)
                    {
                        return;
                    }

                    RaisePropertyChanging(ColorPropertyName);
                    _Color = value;
                    RaisePropertyChanged(ColorPropertyName);
                }
            }

            #endregion

            #region Property Size

            /// <summary>
            /// The <see cref="Size" /> property's name.
            /// </summary>
            public const string SizePropertyName = "Size";

            private double _Size = 5;

            /// <summary>
            /// Sets and gets the Size property.
            /// Changes to that property's value raise the PropertyChanged event. 
            /// </summary>
            public double Size
            {
                get
                {
                    return _Size;
                }

                set
                {
                    if (_Size == value)
                    {
                        return;
                    }

                    RaisePropertyChanging(SizePropertyName);
                    _Size = value;
                    RaisePropertyChanged(SizePropertyName);
                }
            }

            #endregion

            #region Property Style

            /// <summary>
            /// The <see cref="Style" /> property's name.
            /// </summary>
            public const string StylePropertyName = "Style";

            private SimpleLineSymbol.LineStyle _Style = SimpleLineSymbol.LineStyle.Solid;

            /// <summary>
            /// Sets and gets the Style property.
            /// Changes to that property's value raise the PropertyChanged event. 
            /// </summary>
            public SimpleLineSymbol.LineStyle Style
            {
                get
                {
                    return _Style;
                }

                set
                {
                    if (_Style == value)
                    {
                        return;
                    }

                    RaisePropertyChanging(StylePropertyName);
                    _Style = value;
                    RaisePropertyChanged(StylePropertyName);
                }
            }

            #endregion

            #region Command SelectStyleCommand

            private RelayCommand<string> _selectStyleCommand;

            /// <summary>
            /// Gets the SelectStyleCommand.
            /// </summary>
            public RelayCommand<string> SelectStyleCommand
            {
                get
                {
                    return _selectStyleCommand
                        ?? (_selectStyleCommand = new RelayCommand<string>(
                            style => Style = (SimpleLineSymbol.LineStyle)
                                (System.Enum.Parse(typeof(SimpleLineSymbol.LineStyle), style, true)
                                ?? SimpleLineSymbol.LineStyle.Solid),
                            style => (SimpleLineSymbol.LineStyle)
                                System.Enum.Parse(typeof(SimpleLineSymbol.LineStyle), style, true)
                                != Style));
                }
            }

            #endregion

        }

        /// <summary>
        /// This class contains properties that a View can data bind to.
        /// <para>
        /// See http://www.galasoft.ch/mvvm
        /// </para>
        /// </summary>
        public class PolygonViewModel : ViewModelBase
        {

            #region Property FillColor

            /// <summary>
            /// The <see cref="FillColor" /> property's name.
            /// </summary>
            public const string FillColorPropertyName = "FillColor";

            private Color _FillColor = Color.FromArgb(128, 128, 128, 128);

            /// <summary>
            /// Sets and gets the FillColor property.
            /// Changes to that property's value raise the PropertyChanged event. 
            /// </summary>
            public Color FillColor
            {
                get
                {
                    return _FillColor;
                }

                set
                {
                    if (_FillColor == value)
                    {
                        return;
                    }

                    RaisePropertyChanging(FillColorPropertyName);
                    _FillColor = value;
                    RaisePropertyChanged(FillColorPropertyName);
                }
            }

            #endregion

            #region Property BorderColor

            /// <summary>
            /// The <see cref="BorderColor" /> property's name.
            /// </summary>
            public const string BorderColorPropertyName = "BorderColor";

            private Color _BorderColor = Colors.Black;

            /// <summary>
            /// Sets and gets the BorderColor property.
            /// Changes to that property's value raise the PropertyChanged event. 
            /// </summary>
            public Color BorderColor
            {
                get
                {
                    return _BorderColor;
                }

                set
                {
                    if (_BorderColor == value)
                    {
                        return;
                    }

                    RaisePropertyChanging(BorderColorPropertyName);
                    _BorderColor = value;
                    RaisePropertyChanged(BorderColorPropertyName);
                }
            }

            #endregion

            #region Property BorderThickness

            /// <summary>
            /// The <see cref="BorderThickness" /> property's name.
            /// </summary>
            public const string BorderThicknessPropertyName = "BorderThickness";

            private double _BorderThickness = 5;

            /// <summary>
            /// Sets and gets the BorderThickness property.
            /// Changes to that property's value raise the PropertyChanged event. 
            /// </summary>
            public double BorderThickness
            {
                get
                {
                    return _BorderThickness;
                }

                set
                {
                    if (_BorderThickness == value)
                    {
                        return;
                    }

                    RaisePropertyChanging(BorderThicknessPropertyName);
                    _BorderThickness = value;
                    RaisePropertyChanged(BorderThicknessPropertyName);
                }
            }

            #endregion

        }

    }
}
