﻿using GalaSoft.MvvmLight;
using Sigcat.Client.SvcRefSigcatDomOData;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Linq;
using EsriMarkerSymbol = ESRI.ArcGIS.Client.Symbols.SimpleMarkerSymbol;
using EsriSymbol = ESRI.ArcGIS.Client.Symbols.Symbol;

namespace Sigcat.Client.ViewModel
{
    /// <summary>
    /// This class contains properties that a View can data bind to.
    /// <para>
    /// See http://www.galasoft.ch/mvvm
    /// </para>
    /// </summary>
    public class SymbolViewModel : ViewModelBase
    {
        private IDomSvcSigcat svc;

        /// <summary>
        /// Initializes a new instance of the SymbolViewModel class.
        /// </summary>
        public SymbolViewModel(IDomSvcSigcat svc)
        {
            this.svc = svc;
            Symbols = new ObservableCollection<Sigcat.Client.SvcRefSigcatDomOData.Symbol>(svc.Domain.SymbolSet);
            List<Sigcat.Client.SvcRefSigcatDomOData.Building> bs = this.svc.Domain.BuildingSet.ToList();
            Sigcat.Client.SvcRefSigcatDomOData.Building b = new Sigcat.Client.SvcRefSigcatDomOData.Building()
            {
                Name = "Building 1",
                LocationWKT = "POINT (10 10)"
            };
            this.svc.Domain.AddToBuildingSet(b);
            //MarkerSymbol marker = new MarkerSymbol()
            //{
            //    ShortName = "M2",
            //    Name = "Marker 2",
            //    ColorARGB = 241523,
            //    Size = 10,
            //    Style = (byte)EsriMarkerSymbol.SimpleMarkerStyle.Square
            //};
            //this.svc.Domain.AddToSymbolSet(marker);
            //this.svc.Domain.SaveChanges();
            //this.svc.Domain.SaveChanges();
            //Symbols = new ObservableCollection<Symbol>(svc.Domain.SymbolSet.ToList());
        }

        #region Property Symbols

        /// <summary>
        /// The <see cref="Symbols" /> property's name.
        /// </summary>
        public const string SymbolsPropertyName = "Symbols";

        private ObservableCollection<Sigcat.Client.SvcRefSigcatDomOData.Symbol> _Symbols = new ObservableCollection<Sigcat.Client.SvcRefSigcatDomOData.Symbol>();

        /// <summary>
        /// Sets and gets the Symbols property.
        /// Changes to that property's value raise the PropertyChanged event. 
        /// </summary>
        public ObservableCollection<Sigcat.Client.SvcRefSigcatDomOData.Symbol> Symbols
        {
            get
            {
                return _Symbols;
            }

            set
            {
                if (_Symbols == value)
                {
                    return;
                }

                RaisePropertyChanging(SymbolsPropertyName);
                _Symbols = value;
                RaisePropertyChanged(SymbolsPropertyName);
            }
        }

        #endregion

        #region Property EsriSymbols

        /// <summary>
        /// Gets the ESRI Symbols property.
        /// </summary>
        public ObservableCollection<EsriSymbol> EsriSymbols
        {
            get
            {
                return new ObservableCollection<EsriSymbol>(_Symbols.Select(s => s.EsriSymbol));
            }
        }

        #endregion

        #region Property MarkerSymbols

        /// <summary>
        /// Gets the Marker Symbols property.
        /// </summary>
        public ObservableCollection<Sigcat.Client.SvcRefSigcatDomOData.MarkerSymbol> MarkerSymbols
        {
            get
            {
                return new ObservableCollection<Sigcat.Client.SvcRefSigcatDomOData.MarkerSymbol>(_Symbols.Where(s => s is Sigcat.Client.SvcRefSigcatDomOData.MarkerSymbol).Select(s => (Sigcat.Client.SvcRefSigcatDomOData.MarkerSymbol)s));
            }
        }

        #endregion

    }
}
