﻿using Sigcat.Client.Views.WpfWindows.Resources;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Reflection;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Shapes;

namespace Sigcat.Client.Views.WpfWindows.View
{
    /// <summary>
    /// Lógica de interacción para AboutView.xaml
    /// </summary>
    public partial class AboutView : Window
    {
        public struct Descriptor
        {
            public string Property;
            public string Description;
            public bool HiddenProperty;
            public bool WrapDescription;
        }

        public AboutView()
        {
            InitializeComponent();

            Assembly assembly = Assembly.GetExecutingAssembly();
            Descriptor[] descriptors =
            {
                new Descriptor()
                {
                    Property = LiteralResources.Assembly_Title,
                    Description = assembly.GetCustomAttribute<AssemblyTitleAttribute>().Title
                },
                new Descriptor()
                {
                    Property = LiteralResources.Assembly_Version,
                    Description = assembly.GetCustomAttribute<AssemblyFileVersionAttribute>().Version
                },
                new Descriptor()
                {
                    Property = LiteralResources.Assembly_Copyright,
                    Description = assembly.GetCustomAttribute<AssemblyCopyrightAttribute>().Copyright,
                    HiddenProperty = true
                },
                new Descriptor()
                {
                    Property = LiteralResources.Assembly_Company,
                    Description = assembly.GetCustomAttribute<AssemblyCompanyAttribute>().Company
                },
                new Descriptor()
                {
                    Property = LiteralResources.Assembly_Description,
                    Description = assembly.GetCustomAttribute<AssemblyDescriptionAttribute>().Description,
                    WrapDescription = true
                }
            };

            foreach (Descriptor d in descriptors) {
                TextBlock txtBlock = new TextBlock();
                txtBlock.Margin = new Thickness(0, 5, 0, 5);
                if (d.WrapDescription) {
                    stackAttributes.Children.Add(
                        new TextBlock() { Text = string.Format("{0}:", d.Property), Margin = new Thickness(0, 5, 0, 0) }
                    );
                    txtBlock.Text = d.Description;
                    txtBlock.TextWrapping = TextWrapping.Wrap;
                    txtBlock.Margin = new Thickness(0, 0, 0, 5);
                }
                else if (d.HiddenProperty)
                    txtBlock.Text = d.Description;
                else
                    txtBlock.Text = string.Format("{0}: {1}", d.Property, d.Description);
                stackAttributes.Children.Add(txtBlock);
            }
        }

        private void Button_Click(object sender, RoutedEventArgs e)
        {
            Close();
        }
    }
}
