﻿using ESRI.ArcGIS.Client;
using ESRI.ArcGIS.Client.Symbols;
using ESRI.ArcGIS.Client.Toolkit;
using ESRI.ArcGIS.Client.Toolkit.Primitives;
using Sigcat.Client.Model;
using Sigcat.Client.ViewModel;
using System.ComponentModel;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Media;

namespace Sigcat.Client.Views.WpfWindows.View.SpatialView
{
    /// <summary>
    /// Lógica de interacción para EsriMapControlView.xaml
    /// </summary>
    public partial class EsriMapView : UserControl
    {

        private EsriSymbolSelectorViewModel EsriSymbolSelector = null;

        private Draw draw;

        private Symbol _activeSymbol = null;

        public bool IsLogon
        {
            get { return (bool)GetValue(IsLogonProperty); }
            set { SetValue(IsLogonProperty, value); }
        }

        // Using a DependencyProperty as the backing store for IsLogon.  This enables animation, styling, binding, etc...
        public static readonly DependencyProperty IsLogonProperty =
            DependencyProperty.Register("IsLogon", typeof(bool), typeof(EsriMapView), new PropertyMetadata(false));

        public EsriMapView()
        {
            InitializeComponent();
        }

        private void Legend_Refreshed(object sender, Legend.RefreshedEventArgs e)
        {
            LayerItemViewModel removeLayerItemVM = null;

            if (e.LayerItem.LayerItems != null)
            {
                foreach (LayerItemViewModel layerItemVM in e.LayerItem.LayerItems)
                {
                    if (layerItemVM.IsExpanded)
                        layerItemVM.IsExpanded = false;

                    if (layerItemVM.Label == "states")
                        removeLayerItemVM = layerItemVM;
                }

                if (removeLayerItemVM != null)
                    e.LayerItem.LayerItems.Remove(removeLayerItemVM);
            }
            else
            {
                e.LayerItem.IsExpanded = false;
            }
        }

        private void UserControl_Loaded(object sender, RoutedEventArgs e)
        {
            draw = new Draw(MyMap)
            {
                LineSymbol = dock.Resources["DrawLineSymbol"] as LineSymbol,
                FillSymbol = dock.Resources["DrawFillSymbol"] as FillSymbol
            };
            draw.DrawComplete += draw_DrawComplete;

            ViewModelLocator locator = FindResource("Locator") as ViewModelLocator;
            EsriSymbolSelector = locator.EsriSymbolSelector;
            EsriSymbolSelector.PropertyChanged += EsriSymbolSelector_PropertyChanged;

            ChangeDrawSymbol();

            //    List<string> wkt = new List<string>();
            //    wkt.Add(string.Format("POINT({0} {1})", 30, 50));
            //    wkt.Add(string.Format("LINESTRING({0} {1}, {2} {3}, {4} {5}, {6} {7})", 1, 1, 5, 5, 10, 10, 20, 20));
            //    //wkt.Add(string.Format("CIRCULARSTRING({0} {1}, {2} {3}, {4} {5}, {6} {7}, {8} {9})", 1, 1, 2, 0, 2, 0, 2, 0, 1, 1));
            //    wkt.Add(string.Format("POLYGON(({0} {1}, {2} {3}, {4} {5}, {6} {7}, {8} {9}))", -5, -5, -5, 5, 5, 5, 5, -5, -5, -5));
            //    //wkt.Add(string.Format("POLYGON(({0} {1}, {2} {3}, {4} {5}, {6} {7}, {8} {9}),({10} {11}, {12} {13}, {14} {15}, {16} {17}, {18} {19}))", -5, -5, -5, 5, 5, 5, 5, -5, -5, -5, 3, 0, 6, 0, 6, 3, 3, 3, 3, 0));
            //    wkt.Add(string.Format("MULTIPOINT(({0} {1}), ({2} {3}), ({4} {5}))", 2, 3, 7, 8, 9, 5));

            //    string str = "";
            //    foreach (string s in wkt)
            //    {
            //        DbGeography g = DbGeography.FromText(s);
            //        str += Environment.NewLine + g.AsText() + " SpatialTypeName: " + g.SpatialTypeName;
            //    }

            //    //MessageBox.Show(str);
        
        }

        void ChangeDrawSymbol()
        {
            draw.DrawMode = EsriSymbolSelector.Mode;
            draw.LineSymbol = new LineSymbol()
            {
                Color = new SolidColorBrush(EsriSymbolSelector.Line.Color),
                Width = EsriSymbolSelector.Line.Size
            };
            draw.FillSymbol = new FillSymbol()
            {
                BorderThickness = EsriSymbolSelector.Polygon.BorderThickness,
                BorderBrush = new SolidColorBrush(EsriSymbolSelector.Polygon.BorderColor),
                Fill = new SolidColorBrush(EsriSymbolSelector.Polygon.FillColor)
            };
            //switch (EsriSymbolSelector.ModeType)
            //{
            //    case EsriSymbolSelectorViewModel.EsriSymbolModeType.Point:
            //        break;
            //    case EsriSymbolSelectorViewModel.EsriSymbolModeType.Line:
            //        break;
            //    case EsriSymbolSelectorViewModel.EsriSymbolModeType.Polygon:
            //        break;
            //    case EsriSymbolSelectorViewModel.EsriSymbolModeType.None:
            //    default:
            //        break;
            //}
            draw.IsEnabled = draw.DrawMode == DrawMode.None ? false : true;
        }

        void EsriSymbolSelector_PropertyChanged(object sender, PropertyChangedEventArgs e)
        {
            ChangeDrawSymbol();
        }

        private void draw_DrawComplete(object sender, DrawEventArgs args)
        {
            Graphic graphic = new Graphic()
            {
                Geometry = args.Geometry,
                Symbol = _activeSymbol
                // grid.Resources["DrawLineSymbol"] as LineSymbol
            };
            GraphicsLayer graphicsLayer = MyMap.Layers["MyGraphicsLayer"] as GraphicsLayer;
            graphicsLayer.Graphics.Add(graphic);
            MessageBox.Show(GeographyWrapper.GeometryToWKT(GeographyWrapper.GeometryFromWKT(GeographyWrapper.GeometryToWKT(args.Geometry))));
        }

        //private void Toolbar_ToolbarItemClicked(object sender, ESRI.ArcGIS.Client.Toolkit.SelectedToolbarItemArgs e)
        //{
        //    switch (e.Index)
        //    {
        //        case 0: // Point
        //            draw.DrawMode = DrawMode.Point;
        //            _activeSymbol = grid.Resources["DefaultMarkerSymbol"] as Symbol;
        //            break;
        //        case 1: // Polyline
        //            draw.DrawMode = DrawMode.Polyline;
        //            _activeSymbol = grid.Resources["DefaultLineSymbol"] as Symbol;
        //            break;
        //        case 2: // Polygon
        //            draw.DrawMode = DrawMode.Polygon;
        //            _activeSymbol = grid.Resources["DefaultFillSymbol"] as Symbol;
        //            break;
        //        case 3: // Rectangle
        //            draw.DrawMode = DrawMode.Rectangle;
        //            _activeSymbol = grid.Resources["DefaultFillSymbol"] as Symbol;
        //            break;
        //        case 4: // Freehand
        //            draw.DrawMode = DrawMode.Freehand;
        //            _activeSymbol = grid.Resources["DefaultLineSymbol"] as Symbol;
        //            break;
        //        case 5: // Arrow
        //            draw.DrawMode = DrawMode.Arrow;
        //            _activeSymbol = grid.Resources["DefaultFillSymbol"] as Symbol;
        //            break;
        //        case 6: // Triangle
        //            draw.DrawMode = DrawMode.Triangle;
        //            _activeSymbol = grid.Resources["DefaultFillSymbol"] as Symbol;
        //            break;
        //        case 7: // Circle
        //            draw.DrawMode = DrawMode.Circle;
        //            _activeSymbol = grid.Resources["DefaultFillSymbol"] as Symbol;
        //            break;
        //        case 8: // Ellipse
        //            draw.DrawMode = DrawMode.Ellipse;
        //            _activeSymbol = grid.Resources["DefaultFillSymbol"] as Symbol;
        //            break;
        //        default: // Clear Graphics
        //            draw.DrawMode = DrawMode.None;
        //            GraphicsLayer graphicsLayer = MyMap.Layers["MyGraphicsLayer"] as GraphicsLayer;
        //            graphicsLayer.ClearGraphics();
        //            break;
        //    }
        //    draw.IsEnabled = (draw.DrawMode != DrawMode.None);
        //}
    //}

    }
}
