﻿using System.Collections.Generic;
using System.Runtime.Serialization;
using System.ServiceModel.DomainServices.Server.ApplicationServices;

namespace Sigcat.Server.Model
{
    /// <summary>
    /// LoginUser class derives from User class and implements IUser interface,
    /// it only exposes the following three data members to the client:
    /// Name, Password, ProfileResetFlag, and Roles
    /// </summary>
    [DataContractAttribute(IsReference = true)]
    //[MetadataTypeAttribute(typeof(LoginUser.LoginUserMetadata))]
    public sealed class SessionUser : User, IUser
    {

        #region LoginUserMetadata

        //// This class allows you to attach custom attributes to properties
        //// of the LoginUser class.
        ////
        //// For example, the following marks the Xyz property as a
        //// required field and specifies the format for valid values:
        ////    [Required]
        ////    [RegularExpression("[A-Z][A-Za-z0-9]*")]
        ////    [StringLength(32)]
        ////    public string Xyz;
        //internal sealed class LoginUserMetadata : UserMetadata
        //{

        //    // Metadata classes are not meant to be instantiated.
        //    private LoginUserMetadata()
        //    {
        //    }

        //    [Key]
        //    [Display(Name = "UserNameLabel", ResourceType = typeof(MessageResources))]
        //    [Required(ErrorMessageResourceName = "ValidationErrorRequiredField", ErrorMessageResourceType = typeof(ErrorResources))]
        //    [RegularExpression("^[a-zA-Z0-9_]*$", ErrorMessageResourceName = "ValidationErrorInvalidUserName", ErrorMessageResourceType = typeof(ErrorResources))]
        //    public new string Name { get; set; }

        //    [Exclude]
        //    public new string Email { get; set; }

        //    [Exclude]
        //    public string FirstName { get; set; }

        //    [Exclude]
        //    public string LastName { get; set; }

        //    [Exclude]
        //    public string NewPassword { get; set; }

        //    [Exclude]
        //    public string PasswordQuestion { get; set; }

        //    [Exclude]
        //    public string PasswordAnswer { get; set; }

        //    [Exclude]
        //    public string UserType { get; set; }

        //    [Exclude]
        //    public bool IsUserMaintenance { get; set; }
        //}

        #endregion

        #region Property IUser.Name

        /// <summary>
        /// Nom de l'usuari emprat pel sistema d'autentificació.
        /// </summary>
        /// <remarks>
        /// Implementació de la interfície IUser.
        /// </remarks>
        public string Name
        {
            get
            {
                return UserName;
            }
            set
            {
                UserName = value;
            }
        }

        #endregion

        #region Property IUser.Roles

        /// <summary>
        /// Contenidor dels rols del usuari.
        /// </summary>
        private List<string> _roles;

        /// <summary>
        /// Rols de l'usuari emprats pel sistema d'autorització.
        /// </summary>
        /// <remarks>
        /// Implementació de la interfície IUser.
        /// </remarks>
        public IEnumerable<string> Roles
        {
            get
            {
                if (_roles == null)
                    _roles = new List<string>() { Organ.Role.Name };
                else if (_roles[0] != Organ.Role.Name)
                    _roles[0] = Organ.Role.Name;
                return _roles;
            }
            set
            {
                _roles = new List<string>(value);
            }
        }

        #endregion

    }

}
