﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace fgxavi.Mvvm
{

    #region delegate RecieveChangesEventHandler

    /// <summary>
    /// Delegat que encapsula el mètode dels events <see cref="ModelBase.AcceptInsertChanges"/>,
    /// <see cref="ModelBase.AcceptUpdateChanges"/> i <see cref="ModelBase.AcceptDeleteChanges"/>
    /// d'un model de dades <see cref="ModelBase"/>.
    /// </summary>
    /// <param name="sender">
    /// Instàcia del model de dades <see cref="ModelBase"/>
    /// que produeix l'event.
    /// </param>
    public delegate void RecieveChangesEventHandler<T>(T sender, ChangeResultEventArgs<T> args) where T : ModelBase;

    #endregion

    #region delegate SendChangesEventHandler

    /// <summary>
    /// Delegat que encapsula el mètode dels events <see cref="ModelBase.RejectInsertChanges"/>,
    /// <see cref="ModelBase.RejectUpdateChanges"/> i <see cref="ModelBase.RejectDeleteChanges"/>
    /// d'un model de dades <see cref="ModelBase"/>.
    /// </summary>
    /// <param name="sender">
    /// Instàcia del model de dades <see cref="ModelBase"/>
    /// que produeix l'event.
    /// </param>
    public delegate void SendChangesEventHandler<T>(T sender) where T : ModelBase;

    #endregion

    public class ChangeResultEventArgs<T> where T : ModelBase
    {
        public ChangeResult<T> Result { get; set; }
    }

    public class ChangeResult<T> where T : ModelBase
    {
        public ChangeEntityResult<T> EntityResult { get; set; }
        public ChangeErrorResult ErrorResult { get; set; }
        public bool HasError { get; set; }
    }

    public class ChangeEntityResult<T> where T : ModelBase
    {
        public T Entity { get; set; }
        public bool HasMemberChanges { get; set; }
        public int Id { get; set; }
        public int Operation { get; set; }
        public ChangeValidationError[] ValidationErrors { get; set; }
    }

    public class ChangeValidationError
    {
        public string Message { get; set; }
        public string[] SourceMemberNames { get; set; }
    }

    public class ChangeErrorResult
    {
        public bool HasError { get; set; }
        public int ErrorCode { get; set; }
        public string ErrorMessage { get; set; }
        public string ErrorStackTrace { get; set; }
    }

}
