﻿using System.Collections.ObjectModel;

namespace fgxavi.Mvvm
{

    #region delegate SelectionChangedHandler

    /// <summary>
    /// Delegat que encapsula el mètode del event <see cref="ISelectable.SelectionChanged"/>
    /// quan un objecte <see cref="ISelectable"/> es selecciona o es deselecciona.
    /// </summary>
    /// <param name="sender">
    /// Objecte seleccionable <see cref="ISelectable"/> que produeix l'event.
    /// </param>
    /// <param name="selection">
    /// Informa al controlador si s'ha seleccionat l'objecte o si s'ha deseleccionat.
    /// </param>
    public delegate void SelectionChangedHandler(ISelectable sender, bool selection);

    #endregion

    #region Interface ISelectable

    /// <summary>
    /// Interfície que permet implementar
    /// la selecció d'un objecte.
    /// </summary>
    public interface ISelectable
    {

        /// <summary>
        /// Event que desencadenarà l'objecte <see cref="ISelectable"/>
        /// quan es modifiqui la seva selecció.
        /// </summary>
        event SelectionChangedHandler SelectionChanged;

        /// <summary>
        /// Propietat que determina si l'objecte està o no seleccionat.
        /// </summary>
        bool Selected { get; set; }

    }

    #endregion

    #region Interface ICollectionSelectable<T>

    /// <summary>
    /// Interfície que permet implementar
    /// la selecció d'un objecte.
    /// </summary>
    public interface ICollectionSelectable<T>
        where T : ISelectable
    {

        /// <summary>
        /// Propietat que retorna una col·lecció que conté els elements seleccionats.
        /// </summary>
        Collection<T> SelectedItems { get; }

        /// <summary>
        /// Propietat que retorna l'element seleccionat principal.
        /// </summary>
        T SelectedItem { get; }

        /// <summary>
        /// Propietat que retorna si hi ha almenys un element seleccionat.
        /// </summary>
        bool HasSelection { get; }

        /// <summary>
        /// Propietat que retorna si tots els elements estan seleccionats.
        /// </summary>
        bool FullSelection { get; set; }

    }

    #endregion

}
